package com.diagbot.task;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.entity.SysTaskCron;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.facade.BehospitalInfoFacade;
import com.diagbot.facade.MedCheckWorkFacade;
import com.diagbot.facade.SysTaskCronFacade;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.TaskVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.Trigger;
import org.springframework.scheduling.TriggerContext;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.scheduling.support.CronTrigger;

import java.time.LocalDateTime;
import java.util.Date;

@Configuration      //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务(每隔半小时检查今天患者，然后修改今天的患者的类型）
@Slf4j
public class BehospitalInfoTypeTask implements SchedulingConfigurer{
    @Autowired
    private SysTaskCronFacade sysTaskCronFacade;

    private SysTaskCron task001 = new SysTaskCron();

    private String param = "";

    //添加核查任务
    @Autowired
    private MedCheckWorkFacade medCheckWorkFacade;
    /**
     * 执行定时任务.
     */
    @Override
    public void configureTasks(ScheduledTaskRegistrar taskRegistrar) {
        taskRegistrar.addTriggerTask(new Runnable() {
            @Override
            public void run() {
                //1.添加任务内容(Runnable)
                if (null != task001
                        && task001.getIsDeleted().equals(IsDeleteEnum.N.getKey())
                        && task001.getIsUsed().equals(1)) {
                    log.info("执行动态定时任务: " + LocalDateTime.now().toLocalTime());
                    medCheckWorkFacade.execute(param);
                }
            }
        }, new Trigger() {
            @Override
            public Date nextExecutionTime(TriggerContext triggerContext) {
                //2.1 从数据库获取执行周期
                task001 = sysTaskCronFacade.getOne(new QueryWrapper<SysTaskCron>()
                        .eq("cron_code", "TASK019"), false);
                String cron = "0 0/30 * * * ?";
                //2.2 合法性校验.
                if (null != task001 && StringUtil.isNotBlank(task001.getCron())) {
                    cron = task001.getCron();
                    param = task001.getParam();
                }
                CronTrigger trigger = new CronTrigger(cron);
                return trigger.nextExecutionTime(triggerContext);
            }
        });
    }
}
